@@ -13,11 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import, unicode_literals
+from __future__ import absolute_import, division, unicode_literals
 
 import functools
 import inspect
 import json
+import math
 import re
 import sys
 from copy import copy
@@ -134,6 +135,14 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 """Retrieve the appropriate method from the Resource,
                 decorate it as a click command, and return that method.
                 """
+                # Sanity check: Does a method exist corresponding to this
+                # command? If not, this is an error.
+                if not hasattr(self.resource, name):
+                    raise exc.UsageError(
+                        'The %s resource has no such command: "%s"' %
+                        (self.resource_name, name),
+                    )
+
                 # Get the method.
                 method = getattr(self.resource, name)
 
@@ -201,7 +210,7 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 code = six.get_function_code(method)
                 if 'pk' in code.co_varnames:
                     click.argument('pk', nargs=1, required=False,
-                                         type=int)(cmd)
+                                         type=int, metavar='[ID]')(cmd)
 
                 # Done; return the command.
                 return cmd
@@ -217,6 +226,11 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 if not self.resource_name.lower().startswith(an_prefix):
                     help_text = help_text.replace('an object',
                                                   'a %s' % self.resource_name)
+                if self.resource_name.lower().endswith('y'):
+                    help_text = help_text.replace(
+                        'objects',
+                        '%sies' % self.resource_name[:-1],
+                    )
                 help_text = help_text.replace('object', self.resource_name)
 
                 # Convert some common Python terms to their CLI equivalents.
@@ -268,6 +282,9 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                 """Convert the payload into an ASCII table suitable for
                 printing on screen and return it.
                 """
+                page = None
+                total_pages = None
+
                 # What are the columns we will show?
                 columns = [field.name for field in self.resource.fields
                                       if field.display]
@@ -280,9 +297,23 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                         six.text_type(payload['changed']).lower(),
                     )
 
+                # Sanity check: If there is no ID and no results, then this
+                # is unusual output; keep our table formatting, but plow
+                # over the columns-as-keys stuff above.
+                if 'id' not in payload and 'results' not in payload:
+                    columns = [i for i in payload.keys()]
+
                 # Get our raw rows into a standard format.
                 if 'results' in payload:
                     raw_rows = payload['results']
+                    if payload.get('count', 0) > len(payload['results']):
+                        prev = payload.get('previous', 0) or 0
+                        page = prev + 1
+                        count = payload['count']
+                        if payload.get('next', None):
+                            total_pages = math.ceil(count / len(raw_rows))
+                        else:
+                            total_pages = page
                 else:
                     raw_rows = [payload]
 
@@ -333,11 +364,14 @@ class BaseResource(six.with_metaclass(ResourceMeta)):
                     data_rows.append(data_row.rstrip())
 
                 # Result the resulting table.
-                return '\n'.join((
+                response = '\n'.join((
                     divider_row, header_row, divider_row,
                     '\n'.join(data_rows),
                     divider_row,
                 ))
+                if page:
+                    response += '(Page %d of %d.)' % (page, total_pages)
+                return response
 
         return Subcommand(resource=self)
         
@@ -396,11 +430,6 @@ class Resource(BaseResource):
                 raise exc.BadRequest('Attempted to set %s twice.' % query[0])
             kwargs[query[0]] = query[1]
 
-        # If debugging is on, print the URL and data being sent.
-        debug.log('GET %s' % url, fg='blue', bold=True)
-        debug.log('Params: %s' % kwargs, fg='blue', bold=True)
-        debug.log('')
-
         # Make the request to the Ansible Tower API.
         r = client.get(url, params=kwargs)
         resp = r.json()
@@ -471,6 +500,7 @@ class Resource(BaseResource):
             existing_data = self._lookup(
                 fail_on_found=fail_on_found,
                 fail_on_missing=not create_on_missing,
+                include_debug_header=False,
                 **kwargs
             )
             if existing_data:
@@ -525,9 +555,6 @@ class Resource(BaseResource):
 
         # If debugging is on, print the URL and data being sent.
         debug.log('Writing the record.', header='details')
-        debug.log('%s %s' % (method, url), fg='blue', bold=True)
-        debug.log('Data: %s' % kwargs, fg='blue', bold=True)
-        debug.log('')
 
         # Actually perform the write.
         r = getattr(client, method.lower())(url, data=kwargs)
@@ -584,7 +611,8 @@ class Resource(BaseResource):
 
         If the number of results does not equal one, raise an exception.
         """
-        debug.log('Getting the record.', header='details')
+        if kwargs.pop('include_debug_header', True):
+            debug.log('Getting the record.', header='details')
         response = self.read(pk=pk, fail_on_no_results=True,
                              fail_on_multiple_results=True, **kwargs)
         return response['results'][0]
@@ -710,7 +738,8 @@ class Resource(BaseResource):
         r = client.post(url, data={'disassociate': True, 'id': other})
         return {'changed': True}
 
-    def _lookup(self, fail_on_missing=False, fail_on_found=False, **kwargs):
+    def _lookup(self, fail_on_missing=False, fail_on_found=False,
+                      include_debug_header=True, **kwargs):
         """Attempt to perform a lookup that is expected to return a single
         result, and return the record.
 
@@ -733,10 +762,12 @@ class Resource(BaseResource):
 
         # Get the record to write.
         try:
-            existing_data = self.get(**read_params)
+            existing_data = self.get(include_debug_header=include_debug_header,
+                                     **read_params)
             if fail_on_found:
                 raise exc.Found('A record matching %s already exists, and '
-                                'you requested a failure in that case.')
+                                'you requested a failure in that case.' %
+                                read_params)
             return existing_data
         except exc.NotFound as ex:
             if fail_on_missing:
